
package jwiley.chp10;

import java.awt.*;
import java.awt.event.*;
import java.awt.datatransfer.*;

/**
 * This class provides Java copy and paste functionality to
 * java.awt.TextArea
 *
 * @author Eric Monk
 * @version 1.0
 */

public class ExtendedTextArea extends TextArea implements
                                    ClipboardOwner, KeyListener
{
    Clipboard   clipboard;

    public ExtendedTextArea ()
    {
        addKeyListener(this);
        clipboard = getToolkit().getSystemClipboard();
    }

    public void keyPressed (KeyEvent evt)
    {
        if (evt.isControlDown())
        {
            if (evt.getKeyCode() == KeyEvent.VK_A)  // copy command
            {
                String sCopyText = getSelectedText();
                if (sCopyText != null) {
                    StringSelection sClipboardText =
                                new StringSelection(sCopyText);
                    clipboard.setContents(sClipboardText, this);
                }
                evt.consume();
            }
            else if (evt.getKeyCode() == KeyEvent.VK_B) // paste command
            {
                Transferable clipboardData = clipboard.getContents(this);
                if (clipboardData != null)
                {
                    try
                    {
                        String sPasteText =
                                    (String)clipboardData.getTransferData(
                                                    DataFlavor.stringFlavor);
                        insert(sPasteText, getCaretPosition());
                    } catch (Exception e)
                      {
                        System.out.println("Error pasting data");
                      }
                }
                evt.consume();
            }
        }
    }

    public void keyReleased (KeyEvent evt)  {   }
    public void keyTyped (KeyEvent evt)     {   }

    public void lostOwnership(Clipboard clipboard, Transferable contents)
    {   }

    public static void main (String args[])
    {
        Frame f = new Frame("Test Clipboard");
        f.setLayout(new GridLayout(0,1));

        ExtendedTextArea textArea1 = new ExtendedTextArea();
        ExtendedTextArea textArea2 = new ExtendedTextArea();
        f.add(textArea1);
        f.add(textArea2);

        f.setSize(300,300);
        f.show();
    }

}