// 
package jwiley.chp3;
/**
 *  LayoutTester.java
 *  A class that demonstrates the basic AWT Layout objects.
 *  @version 1.1
 *  @author M. Daconta.
 */
import java.awt.*;
import java.awt.event.*;

/** Class to demonstrate the built-in layout managers. */
class LayoutTester extends Frame implements ActionListener
{
    /** buttons to lay out. */
    Button buttons[];
    /** card layout.*/
    CardLayout cl = new CardLayout();

    /** constructor. */
    LayoutTester()
    {
        super("Layout Tester");

        buttons = new Button[5];
        buttons[0] = new Button("One");
        buttons[1] = new Button("Two");
        buttons[2] = new Button("Three");
        buttons[3] = new Button("Four");
        buttons[4] = new Button("Five");

        // start with border Layout
        setLayout(new BorderLayout());
        add("North",buttons[0]);
        add("Center",buttons[1]);
        add("South",buttons[2]);
        add("West",buttons[3]);
        add("East",buttons[4]);

        // add the MenuBar
        MenuBar mBar = new MenuBar();
        setMenuBar(mBar);
        Menu fileMenu = new Menu("File");
        mBar.add(fileMenu);
        Menu layoutMenu = new Menu("Layouts");
        mBar.add(layoutMenu);
        MenuItem emi = new MenuItem("Exit");
        emi.addActionListener(this);
        fileMenu.add(emi);

        MenuItem mi = new MenuItem("BorderLayout");
        mi.addActionListener(this);
        layoutMenu.add(mi);
        mi = new MenuItem("FlowLayout");
        mi.addActionListener(this);
        layoutMenu.add(mi);
        mi = new MenuItem("GridLayout");
        mi.addActionListener(this);
        layoutMenu.add(mi);
        mi = new MenuItem("GridBagLayout");
        mi.addActionListener(this);
        layoutMenu.add(mi);
        mi = new MenuItem("CardLayout");
        mi.addActionListener(this);
        layoutMenu.add(mi);

        setSize(300,300);
        setLocation(50,50);
        setVisible(true);
    }

    /** ActionListener method. */
    public void actionPerformed(ActionEvent evt)
    {
        String label = evt.getActionCommand();
        if (label.equals("Exit"))
        {
            dispose();
            System.exit(0);
        }
        else if (label.equals("BorderLayout"))
        {
            removeAll();
            setLayout(new BorderLayout());
            add("North",buttons[0]);
            add("Center",buttons[1]);
            add("South",buttons[2]);
            add("West",buttons[3]);
            add("East",buttons[4]);
            doLayout();
            repaint();
        }
        else if (label.equals("FlowLayout"))
        {
            removeAll();
            setLayout(new FlowLayout(FlowLayout.LEFT));
            add(buttons[0]);
            add(buttons[1]);
            add(buttons[2]);
            add(buttons[3]);
            add(buttons[4]);
            doLayout();
            repaint();
        }
        else if (label.equals("GridLayout"))
        {
            removeAll();
            setLayout(new GridLayout(0,2));
            add(buttons[0]);
            add(buttons[1]);
            add(buttons[2]);
            add(buttons[3]);
            add(buttons[4]);
            doLayout();
            repaint();
        }
        else if (label.equals("GridBagLayout"))
        {
            removeAll();
            GridBagLayout gbl = new GridBagLayout();
            setLayout(gbl);

            GridBagConstraints c[] = new GridBagConstraints[5];
            for (int i=0; i < 5; i++)
                    c[i] = new GridBagConstraints();

            c[0].gridwidth = 1;
            c[0].gridheight = 1;
            c[0].gridx = 0;
            c[0].gridy = 0;
            add(buttons[0],c[0]);

            c[1].gridwidth = 1;
            c[1].gridheight = 1;
            c[1].gridx = 1;
            c[1].gridy = 1;
            add(buttons[1],c[1]);

            c[2].gridwidth = 1;
            c[2].gridheight = 1;
            c[2].gridx = 2;
            c[2].gridy = 2;
            add(buttons[2],c[2]);

            c[3].gridwidth = 1;
            c[3].gridheight = 1;
            c[3].gridx = 3;
            c[3].gridy = 3;
            add(buttons[3],c[3]);

            c[4].gridwidth = 1;
            c[4].gridheight = 1;
            c[4].gridx = 4;
            c[4].gridy = 4;
            add(buttons[4],c[4]);
            doLayout();
            repaint();
        }
        else if (label.equals("CardLayout"))
        {
            removeAll();
            setLayout(cl);
            Panel p[] = new Panel[5];
            for (int i=0; i < 5; i++)
            {
                    p[i] = new Panel();
                    p[i].setLayout(new BorderLayout());
            }

            Button nextButton = new Button("Next");
            nextButton.addActionListener(this);
            p[0].add("Center",buttons[0]);
            p[0].add("South", nextButton);
            p[1].add("Center",buttons[1]);
            Button nextButton1 = new Button("Next");
            nextButton1.addActionListener(this);
            p[1].add("South", nextButton1);
            p[2].add("Center",buttons[2]);
            Button nextButton2 = new Button("Next");
            nextButton2.addActionListener(this);
            p[2].add("South", nextButton2);
            p[3].add("Center",buttons[3]);
            Button nextButton3 = new Button("Next");
            nextButton3.addActionListener(this);
            p[3].add("South", nextButton3);
            p[4].add("Center",buttons[4]);
            Button nextButton4 = new Button("Next");
            nextButton4.addActionListener(this);
            p[4].add("South", nextButton4);

            add("One",p[0]);
            add("Two",p[1]);
            add("Three",p[2]);
            add("Four",p[3]);
            add("Five",p[4]);
            cl.first(this);
            doLayout();
            repaint();
        }
        else if (label.equals("Next"))
        {
            // card Layout - next card
            cl.next(this);
        }
    }

    /** main() method to invoke from the JVM. */
    public static void main(String args[])
    {
        new LayoutTester();
    }
}
