// URLDisplay.java
package jwiley.chp11;

import java.awt.*;
import java.awt.image.*;
import java.net.URL;
import java.net.MalformedURLException;
import java.util.Vector;
import java.io.BufferedInputStream;
import java.io.IOException;

/**
 * Java bean that displays URLs.  This class is very simplistic
 * it attempts to display everything as text, regardless of the
 * content.  This is a visual Bean, so it needs to extend an AWT
 * component class
 */
public class URLDisplay extends Canvas
{
    /**
     * value of the Display property
     */
    private boolean bDisplay = true;

    /**
     * value of the URL property
     */
    private String strLocation = "";

    /**
     * URL object.  This only exists if the URL property
     * is properly formatted
     */
    private URL url = null;

    /**
     * Object referred to by the URL.  This only exists if the URL
     * object exists, and it pointed to a valid location
     */
    private Object curObj = null;

    /**
     * A vector of all the listeners for our event
     */
    private Vector listeners = new Vector();

    /**
     * Read the Display property
     */
    public synchronized boolean getDisplay()
    {
        return bDisplay;
    }

    /**
     * Write the Display property
     */
    public synchronized void setDisplay(boolean bNewVal)
    {
        bDisplay = bNewVal;
    }

    /**
     * Read the URL property
     */
    public synchronized String getURL()
    {
        return strLocation;
    }

    /**
     * Write the URL property
     */
    public synchronized void setURL(String newStr)
    {
        strLocation = newStr;
        reload();
    }

    /**
     * Load (or reload) the object pointed to by the URL
     */
    public synchronized void reload()
    {
        curObj = "Loading...";

        try
        {
            url = new URL(strLocation);
            curObj = url.getContent();
            if (curObj instanceof BufferedInputStream)
            {
                boolean bStore = true;
                StringBuffer sb = new StringBuffer();
                while (((BufferedInputStream)curObj).available() > 0)
                {
                    char c = (char)((BufferedInputStream)curObj).read();
                    if (!bStore && c == '>')
                    {
                        bStore = true;
                        continue;
                    }
                    if (c == '<')
                    {
                        bStore = false;
                        continue;
                    }
                    if (bStore)
                    {
                        sb.append(c);
                    }
                }
                curObj = sb.toString();
            }
            else if (curObj instanceof ImageProducer)
            {
                curObj = Toolkit.getDefaultToolkit().createImage((ImageProducer)curObj);
            }
        }
        catch (java.io.IOException iox)
          {
            curObj = null;
          }

        fireEvent(curObj);
        repaint();
    }

    // java.awt.Component support
    /**
     * Paint the object if we have it.  Paint an error
     * message otherwise.
     */
    public synchronized void paint(Graphics g)
    {
        if (bDisplay)
        {
            if (curObj == null)
            {
                if (url == null)
                {
                    g.drawString("No URL specified", 10, 10);
                }
                else
                {
                    g.drawString("There was an error loading the URL",
                                 10, 10);
                }
            }
            else if (curObj instanceof String)
            {
                g.drawString((String)curObj, 10, 10);
            }
            else if (curObj instanceof Image)
            {
                g.drawImage((Image)curObj,0,0,this);
            }
            else
            {
                g.drawString("Could not display URL", 10, 10);
            }
        }
    }

    // event support
    /**
     * Add a listener for our event
     */
    public synchronized void addURLLoadedListener(URLLoadedListener l)
    {
        listeners.addElement(l);
    }

    /**
     * Remove a listener for our event
     */
    public synchronized void removeURLLoadedListener(URLLoadedListener l)
    {
        listeners.removeElement(l);
    }

    /**
     * Fire our event to signify that we are done loading the URL
     */
    public synchronized void fireEvent(Object obj)
    {
        URLLoadedEvent event = new URLLoadedEvent(this, obj);
        for (int i = 0; i < listeners.size(); i++)
        {
            ((URLLoadedListener)listeners.elementAt(i)).URLLoaded(event);
        }
    }
}
