// URLDisplayCustomizer.java
package jwiley.chp11;

import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.beans.*;

/**
 * Customizer for the URLDisplay Bean.  This is a simple panel which
 * allows the user to input the URL through an alternative mechanism
 */
public class URLDisplayCustomizer extends Panel implements Customizer, ActionListener
{
    /**
     * Instance of the URLDisplay bean being editted
     */
    URLDisplay bean;

    /**
     * Helper class for handling PropertyChangeEvents
     */
    private PropertyChangeSupport propChange = new PropertyChangeSupport(this);

    /**
     * The TextField object on the panel.  This is used to
     * retrieve the new value of the URL property
     */
    TextField text = null;

    /**
     * Default constructor
     */
    public URLDisplayCustomizer()
    {
        setLayout(new BorderLayout());
    }

    /**
     * A Customizer method. This is called to associate a Bean instance
     * with a customizer instance
     */
    public void setObject(Object obj)
    {
    	bean = (URLDisplay) obj;

    	Label label = new Label("  Enter a URL:");
    	add("North", label);

    	text = new TextField(bean.getURL());
    	add("Center", text);

    	text.addActionListener(this);
    }

    /**
     * The size the customizer should be
     */
    public Dimension getPreferredSize() {
    	return new Dimension(300, 50);
    }

    /**
     * AWT event handler so we know when the TextField is modified
     */
    public void actionPerformed(ActionEvent e)
    {
        bean.setURL(text.getText());
    }

    /**
     * Customizer event support
     */
    public void addPropertyChangeListener(PropertyChangeListener listener)
    {
	    propChange.addPropertyChangeListener(listener);
    }

    /**
     * Customizer event support
     */
    public void removePropertyChangeListener(PropertyChangeListener listener)
    {
	    propChange.removePropertyChangeListener(listener);
    }
}