// ScrollMsg.java
import java.awt.*;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

/** Class to create a scrolling marquee. */
class ScrollMsg extends Panel implements Runnable
{
    /** message to display. */
    String theMessage;
    /** Font to display text in. */
    Font textFont;
    /** Font metrics. */
    FontMetrics fontMetrics;
    /** # of pixels to shift. */
    int stepPixels;
    /** coordinates. */
    int oldX, oldY, newX, newY;
    /** milliseconds between repaint(). */
    int speed; // in milliseconds

    /** constructor. */
    ScrollMsg(String msg)
    {
        setBackground(Color.lightGray);

        theMessage = msg;

        textFont = new Font("TimesRoman",Font.BOLD, 18);

        stepPixels = 6;
        speed = 15;
    }

    /** method to implement runnable. */
    public void run()
    {
        paint(this.getGraphics());
        while (true)
                repaint(speed);
    }

    /** paint method to paint background. */
    public void paint(Graphics g)
    {
        // initial background
        g.setColor(Color.black);
        g.drawRect(0,0, getSize().width,
                   getSize().height);
        g.setColor(Color.white);
        g.fillRect(1,1, getSize().width - 1,
                   getSize().height - 1);

        // set the font
        g.setFont(textFont);
        fontMetrics = g.getFontMetrics();

        // set initial X and Y to drawString
        oldX = newX = getSize().width -
                fontMetrics.charWidth(theMessage.charAt(0));
        oldY = newY = getSize().height/2 +
                (fontMetrics.getHeight()/2);
    }

    /** method to paint the foreground. */
    public void update(Graphics g)
    {
        g.setFont(textFont);

        // Erase the old
        g.setColor(Color.white);
        g.drawString(theMessage, oldX, oldY);

        // draw the New
        g.setColor(Color.red);
        g.drawString(theMessage, newX, newY);

        // step
        oldX = newX;
        newX -= stepPixels;

        // restart?
        if (newX <= -(fontMetrics.stringWidth(theMessage)
                      - fontMetrics.charWidth(
                        theMessage.charAt(
                           theMessage.length() - 1))))
                paint(g);  // start at the beginning
    }

    /** main() method to invoke from the JVM. */
    public static void main(String args[])
    {
        if (args.length < 1)
        {
            System.out.println("USAGE: java ScrollMsg \"textMessage\"");
            System.exit(1);
        }

        Frame myFrame = new Frame("Scrolling Message");
        ScrollMsg myScroll =
                new ScrollMsg(args[0]);
        myFrame.add("Center", myScroll);
        myFrame.setLocation(100,100);
        myFrame.setSize(400,100);
        myFrame.setVisible(true);
        myFrame.addWindowListener(new WindowAdapter()
                                  {
                                    public void windowClosing(WindowEvent we)
                                    { System.exit(0);}
                                  });

        Thread animateThread = new Thread(myScroll);
        animateThread.start();
    }
}
