// SimpleMarquee.java
package jwiley.chp12;

import java.applet.Applet;
import java.awt.*;

/** class to implement same functionality as the ScrollMsg
    class but as an applet. */
public class SimpleMarquee extends Applet implements Runnable
{
    /** message to scroll. */
    String theMessage;
    /** font. */
    Font textFont;
    /** metrics of the font. */
    FontMetrics fontMetrics;
    /** number of pixels to move. */
    int stepPixels;
    /** coordinates. */
    int oldX, oldY, newX, newY;
    /** repaint() speed. */
    int speed; // in milliseconds
    /** Thread to scroll in. */
    Thread scroller=null;

    /** standard applet methods. */
    public void init()
    {
        theMessage = getParameter("message");
        if (theMessage == null)
                theMessage = new
                        String("A simple Marquee by Michael Daconta.");

        String speedStr = getParameter("speed");
        if (speedStr == null)
                speed = 15;
        else
                speed = Integer.parseInt(speedStr);

        textFont = new Font("TimesRoman",Font.BOLD, 18);
        stepPixels = 6;
    }

    /** applet method. */
    public void start()
    {
        if (scroller == null)
        {
            scroller = new Thread(this);
            scroller.start();
        }
    }

    /** applet method. */
    public void stop()
    {
        if (scroller != null)
        {
            scroller.stop();
            scroller = null;
        }
    }

    /** method to implement runnable. */
    public void run()
    {
            Thread.currentThread().setPriority(Thread.NORM_PRIORITY-1);
            paint(this.getGraphics());
            while (true)
                    repaint(speed);
    }

    /** method to paint the background. */
    public void paint(Graphics g)
    {
        // initial background
        g.setColor(Color.black);
        g.drawRect(0,0, getSize().width,
                   getSize().height);
        g.setColor(Color.white);
        g.fillRect(1,1, getSize().width - 1,
                   getSize().height - 1);

        // set the font
        g.setFont(textFont);
        fontMetrics = g.getFontMetrics();

        // set initial X and Y to drawString
        oldX = newX = getSize().width -
                fontMetrics.charWidth(theMessage.charAt(0));
        oldY = newY = getSize().height/2 +
                (fontMetrics.getHeight()/2);
    }

    /** method to paint the foreground. */
    public void update(Graphics g)
    {
        g.setFont(textFont);

        // Erase the old
        g.setColor(Color.white);
        g.drawString(theMessage, oldX, oldY);

        // draw the New
        g.setColor(Color.red);
        g.drawString(theMessage, newX, newY);

        // step
        oldX = newX;
        newX -= stepPixels;

        // restart?
        if (newX <= -(fontMetrics.stringWidth(theMessage)
                      - fontMetrics.charWidth(
                        theMessage.charAt(
                           theMessage.length() - 1))))
                paint(g);  // start at the beginning
    }
}
