
import java.io.*;
import java.security.*;


/**
 * This class takes an algorithm and a file name and
 * generates a message digest based on the combination.
 * For instructional purposes the use of the message digest functions
 * was broken into three parts.
 *		Creating the digest
 *		Updating the digest with the file contents
 *      and then
 *		Generating the result.
 * <pre>
 *		CalculateDigest digest = newCalculateDigest("MD5","Somefile");
 *		digest.Calculate(); 
 *	    byte[] result = digest.Result();
 * </pre>
 *  @version 1.1
 *  @author A. Saganich
 *  @see java.Security.MessageDigest
*
 */
class CalculateDigest 
{
	String 				algorithm;
	FileInputStream   	file;
	MessageDigest  		md;
	
	CalculateDigest(String inalgorithm, String infileName)
	throws FileNotFoundException
	{
		algorithm = inalgorithm;
		// May throw file not found.
		file = new FileInputStream(infileName);
	}

	public void Calculate() throws NoSuchAlgorithmException, IOException
	{
		byte data[] = new byte[512];
		int ctRead;
		md = MessageDigest.getInstance(algorithm);
		while(true)
		{
			ctRead = file.read(data);
			if ( ctRead <= 0) break;
			md.update(data);
		}
	}
	public byte[] Result()
	{
		return md.digest();
	}
}
