// Base16.java
package jwiley.chp2;

import java.io.*;
import java.net.Socket;
import java.net.*;

/**
 * Class to convert from binary and text format.
 * @version 1.1
 * @author Michael C. Daconta
 */
class base16Encoder
{
    /** global constant indicating whether
        or not we are in debug mode */
    public static final boolean debug = false;

    /** global encoding beteen binary
        and text characters */
    static final char encoding[] = { '0', '1', '2', '3', '4',
                               '5', '6', '7', '8', '9',
                               'A', 'B', 'C', 'D', 'E',
                               'F' };

    /**
     * Converts from binary to text
     */
    public static void encodeStream(InputStream is, OutputStream os) throws IOException
    {
        DataInputStream dis = new DataInputStream(is);
        PrintStream ps = new PrintStream(os);
        int lineLength=0;
        long byteCount=0;
        long charCount=0;
        try
        {
            while (true)
            {
                byte b = dis.readByte();
                byteCount++;
                int upper = (b >> 4) & 0xf;
                int lower = (b & 0xf);

                ps.write(encoding[upper]); // writes a byte (not char)
                ps.write(encoding[lower]);
                charCount += 2;
                lineLength += 2;
                if (lineLength >= 76)
                {
                    ps.write('\n');
                    lineLength = 0;
                }
            }
        } catch (EOFException eof)
          {
            // OK, do nothing
            if (debug)
            {
                System.out.println("base16Encoder, byteCount: " + byteCount);
                System.out.println("base16Encoder, charCount: " + charCount);
            }
          }
          catch (IOException ioe)
          {
            throw new IOException(ioe.getMessage());
          }
    }
}

/**
 * Class to convert from text and binary format.
 * @version 1.1
 * @author Michael C. Daconta
 */
class base16Decoder
{
    /** global constant indicating whether
        or not we are in debug mode */
    public static final boolean debug = false;

    /**
     * Method that converts a character to binary
     */
    private static int decodeChar(int ch)
    {
        if ( (ch >= '0') && (ch <= '9') )
            return ch - '0';
        else
            return ch - 'A' + 10;
    }

    /**
     * Method that converts from text to binary
     */
    public static void decodeStream(InputStream is, OutputStream os) throws IOException
    {
        DataInputStream dis = new DataInputStream(is);
        DataOutputStream dos = new DataOutputStream(os);
        int lineLength=0;
        long charCount = 0;
        long byteCount = 0;
        try
        {
            while (true)
            {
                char upperC = (char) dis.readByte();
                if (upperC == '\n')
                    continue;
                char lowerC = (char) dis.readByte();
                charCount+=2;
                int upperB = decodeChar(upperC);
                int lowerB = decodeChar(lowerC);
                byte b = (byte) (upperB << 4);
                b |= lowerB;
                dos.writeByte(b);
                byteCount++;
            }
        } catch (EOFException eof)
          {
            // OK, do nothing
            if (debug)
            {
                System.out.println("base16Decoder, charCount: " + charCount);
                System.out.println("base16Decoder, byteCount: " + byteCount);
            }
          }
          catch (IOException ioe)
          {
            throw new IOException(ioe.getMessage());
          }
    }
}

/**
 * Class to convert between text and binary formats.
 * @version 1.1
 * @author Michael C. Daconta
 */
class Base16
{
    /**
     * Main method to be called by JVM
     */
    public static void main(String args[])
    {
        if (args.length < 2)
        {
            System.out.println("USAGE: java base16 inputFile outputFile");
            System.exit(1);
        }

        String inputFileName = args[0];
        String outputFileName = args[1];

        try
        {
            // encode
            FileInputStream fis = new FileInputStream(inputFileName);
            int dotIdx = inputFileName.indexOf('.');
            String tmpFileName = inputFileName.substring(0,dotIdx) + ".enc16";
            System.out.println("Writing: " + tmpFileName);
            FileOutputStream tmpFos = new FileOutputStream(tmpFileName);
            base16Encoder.encodeStream(fis,tmpFos);
            fis.close();
            tmpFos.close();

            // decode
            fis = new FileInputStream(tmpFileName);
            FileOutputStream fos = new FileOutputStream(outputFileName);
            base16Decoder.decodeStream(fis,fos);
            fis.close();
            fos.close();
        } catch (Exception e)
          {
            e.printStackTrace();
            System.exit(1);
          }
    }
}
