// GreyScaleTest.java
package jwiley.chp2;

import java.awt.image.*;
import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.Toolkit;
import java.awt.Frame;

/**
 * Class to demonstrate break to a label by way of
 * testing whether an image is greyscale or color.
 * @author Michael C. Daconta
 */
class GreyScaleTest
{
    /** Main method to invoke from JVM. */
    public static void main(String args[])
    {
        if (args.length < 1)
        {
            System.out.println("USAGE: java GreyScaleTest <imgFileName>");
            System.exit(1);
        }

        String imgTitle = args[0];
        boolean isGreyScale = true; // optimist
        Image img1=null, img2=null;

        /* load gifs into program. */
        img1 = (Toolkit.getDefaultToolkit()).getImage(imgTitle);
        System.out.println("Loading " + imgTitle + "...");

        /* wait until image is fully loaded. */
        Frame dummyFrame = new Frame();
        MediaTracker imgTracker = new MediaTracker(dummyFrame);
        imgTracker.addImage(img1,0);
        try
        {
            imgTracker.waitForAll();
        } catch (InterruptedException ie)
          {
            System.out.println(ie.getMessage());
            System.exit(1);
          }
        if (imgTracker.isErrorAny())
        {
            System.out.println("Error loading image.");
            System.exit(1);
        }

        /* get height and width of image. */
        int imgWidth = img1.getWidth(dummyFrame);
        int imgHeight = img1.getHeight(dummyFrame);
        System.out.println("imgWidth : " + imgWidth);
        System.out.println("imgHeight: " + imgHeight);

        /* create an array of integers to hold the pixels */
        int [] pixels = new int[imgWidth * imgHeight];

        /* now image is loaded, extract pixels. */
        PixelGrabber pg = new PixelGrabber(img1,0,0,imgWidth,imgHeight,
                                           pixels, 0, imgWidth);

        try
        {
            pg.grabPixels();
        } catch (InterruptedException ie)
          {
            System.out.println(ie.getMessage());
            System.exit(1);
          }

        outerLoopLabel:
        for (int i=0; i < imgHeight; i++)
        {
            for (int j=0; j < imgWidth; j++)
            {
                int pixel = pixels[(i * imgHeight) + j];
                int red   = (pixel >> 16) & 0xff;
                int green = (pixel >>  8) & 0xff;
                int blue  = (pixel      ) & 0xff;

                if (red != green || red != blue || blue != green)
                {
                    isGreyScale = false;
                    break outerLoopLabel;
                }
            } // end of fork(int j=...
        } // end of for (int i= ...

        if (isGreyScale)
            System.out.println("Image is a greyScale image.");
        else
            System.out.println("Image is a color image.");

        System.exit(0);
    }
}