package jwiley.chp21;

import java.applet.*;		// For applet
import java.awt.*;			// for layout etc
import java.net.*;			// for URL
import java.io.IOException;	// for IO Exceptions
import java.media.*;		// for the Java media class

/**
 *  Player2.java
 *  A class that creates a media player using the default controls
     which plays a clip in an infinite loop
 *  @version 1.0
 *  @author A.J. Saganich Jr.
 */
public class Player2 
	extends Applet 
	implements ControllerListener
	//  requires controllerUpdate(ControllerEvent event) be implemented
{
	Player 		aPlayer 	= null;  
	Component 	vc 			= null;
	Component	controls	= null;
	boolean 	fRunning 	= false;		// Not initially running

	/**
	 * Read the name of the media file and then begin processing
	 */
	public void init()
	{
		String inputFile = null;
		URL inputURL	 = null;

		// Use a border based layout
		setLayout(new BorderLayout());

		/**
		 * the inputFile parameter contains the name
		 * of the media file to play.
		 */
		inputFile = getParameter("inputFile");
		if ( null == inputFile)
		{
			System.out.println("Invalid media file parameter");
			return;
		}

		System.out.println("Running on file: "+ inputFile);
		try
		{
			// Create an url from the file name and the url to the
			// document containing this applet.
			inputURL = new URL(getDocumentBase(), inputFile);
			// Create a player that corresponds to the input type.
			aPlayer = Manager.createPlayer(inputURL);
			// If the player was created then we want 
			// to listen to its events.
			if(aPlayer != null)
			{
				aPlayer.addControllerListener(this);
			}
			else
			{
				System.out.println("Could not create aPlayer for " + inputURL);
			}
		}
		catch (MalformedURLException e)
		{
			System.out.println("Invalid media file URL!");
		}
		catch(IOException e)
		{
			System.out.println("IO exception creating aPlayer for" + inputURL);
		}
	}

	/**
	 * Start method
	 * Note that this method attempts to start the prefetch of the
	 * clip but does not start the player playing.  When the prefetch
	 * completes the a event will be generated and the controllerUpdate
	 * method called which will start the clip playing.
	 */
	public void start()
	{
		if (aPlayer != null)
			aPlayer.prefetch();
	}

	/**
	 * Stop method
	 * stop the player playing, if allocated, and release its resources.
	 */
	public void stop()
	{
		if (aPlayer != null)
		{
			fRunning = false;
			aPlayer.stop();
			aPlayer.deallocate();
		}
	}

	/**
	 * controllerUpdate method
	 * required by the ControlllerLister interface
	 * called whenever a media event occurs
	 */
	public synchronized void controllerUpdate(ControllerEvent event)
	{
		System.out.println("ControllerUpdate entered!");
		System.out.println("\twith event " + event.toString());
		// When the aPlayer is Realized, get the visual component
		// and add it to the Applet
		if (event instanceof RealizeCompleteEvent)
		{
			vc = aPlayer.getVisualComponent();
			if (vc != null)
				add("Center", vc);
			
			controls = aPlayer.getControlPanelComponent();
			if ( controls != null)
			{	
				controls.setBackground(Color.lightGray);
				if ( vc != null )
					add("South", controls);
				else
					add("Center", controls); // No visual component must be a sound file.

			}

			// force the applet to draw the image and the controls
			validate();
		}
		else if (event instanceof PrefetchCompleteEvent)
		{
			aPlayer.start();
			fRunning = true;
		}
		else if (event instanceof EndOfMediaEvent)
		{
			aPlayer.setMediaTime(0);
			if ( true == fRunning)
				aPlayer.start();
		}
		else
		{
			// Some other event occured
		}
	} // End method controllerUpdate
} // End class Player2
