/** SimplePanel.java */
package jwiley.chp23;

import java.awt.*;
import java.awt.event.*;
import com.sun.java.swing.*;

/**
 * An application that displays a JButton and two JRadioButtons.
 * The JRadioButtons determine the look and feel used by the application.
 */
public class SimplePanel extends JPanel {
    static JFrame frame;
    static String basic = new String("Win95");
    static String rose = new String("Rose");

    /** Constructor. */
    public SimplePanel()
	{
	    super(true);

    	// start out with Win95 UIFactory
        try
        {
            UIManager.setUIFactory("com.sun.java.swing.basic.BasicFactory",
                                   (Container)frame);
        } catch (ClassNotFoundException cnf)
          {
            System.err.println("Could not load Win95 factory.");
          }

        setLayout(new BorderLayout());
	    setBackground(Color.lightGray);

    	// Create a button.
    	JButton button = new JButton("Hello, world");
            button.setKeyAccelerator('h');

        // Create the glyphs for radio buttons
        ImageGlyph radio = new ImageGlyph("jwiley\\chp23\\offbut.jpg");
        ImageGlyph radioSelected = new ImageGlyph("jwiley\\chp23\\onbut.jpg");

        // Create some radio buttons with image glyphs
        JRadioButton testButton = new JRadioButton("Radio1");
        testButton.setSelected(true);
        JRadioButton test2Button = new JRadioButton("Radio2");
        ButtonGroup testGroup = new ButtonGroup();
        testGroup.add(testButton);
        testGroup.add(test2Button);

    	JRadioButton basicButton = new JRadioButton(basic, radio);
        basicButton.setKeyAccelerator('b');
    	basicButton.setActionCommand(basic);
    	basicButton.setSelected(true);
    	basicButton.setSelectedGlyph(radioSelected);

    	JRadioButton roseButton = new JRadioButton(rose, radio);
            roseButton.setKeyAccelerator('r');
    	roseButton.setActionCommand(rose);
    	roseButton.setSelectedGlyph(radioSelected);

    	// Group the radio buttons.
    	ButtonGroup group = new ButtonGroup();
    	group.add(basicButton);
    	group.add(roseButton);

        // Register a listener for the radio buttons.
    	RadioListener myListener = new RadioListener();
    	roseButton.addActionListener(myListener);
    	basicButton.addActionListener(myListener);

        JPanel centerP = new JPanel();
        centerP.setLayout(new FlowLayout(FlowLayout.LEFT,5,5));
        JPanel southP = new JPanel();

    	centerP.add(button);
    	centerP.add(testButton);
    	centerP.add(test2Button);

    	// controls
    	southP.add(basicButton);
    	southP.add(roseButton);

        // add Panels
        add("Center", centerP);
        add("South", southP);
    }


    /** An ActionListener that listens to the radio buttons. */
    class RadioListener implements ActionListener
    {
	    public void actionPerformed(ActionEvent e)
	    {
	        String factoryName = null;

	        if (e.getActionCommand() == rose)
	            factoryName = "com.sun.java.swing.rose.RoseFactory";
	        else
	            factoryName = "com.sun.java.swing.basic.BasicFactory";

            try
            {
                UIManager.setUIFactory(factoryName, (Container)frame);
		        frame.pack();
            } catch (ClassNotFoundException cnf)
              {
                    System.err.println("Could not load factory: "
				                        + factoryName);
              }
	    }
    }

    /** main() method to invoke from JVM. */
    public static void main(String args[])
    {
         WindowListener l = new WindowAdapter()
         {
             public void windowClosing(WindowEvent e)
             {System.exit(0);}
         };

         frame = new JFrame("Simple Example");
         frame.setLayout(new BorderLayout(5,5));
         frame.addWindowListener(l);
         frame.add("Center", new SimplePanel());
         frame.pack();
         frame.setLocation(50,50);
         frame.setVisible(true);
    }
}
