// DrawShapes.java
package jwiley.chp3;

import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.lang.Math;
import java.util.Random;

/**
 * Abstract class representing a generic shape.
 */
abstract class Shape
{
    /** Number of points in shape. */
    protected Point pts[];
    /** Center of shape. */
    protected Point Center;

    /** abstract method, MUST be implemented in subclasses. */
    abstract void calculatePoints();
}

/**
 * Subclass of shape. Represents a square.
 */
class Square extends Shape
{
    /** the drawing window. */
    drawWindow dw;
    /** The width of the square. */
    int theWidth;

    /** constructor. */
    Square(drawWindow win, Point center, int width)
    {
        dw = win;
        pts = new Point[4];
        Center = center;
        theWidth = width;

        for (int i=0; i < pts.length; i++)
                pts[i] = new Point(0,0);
    }

    /** Method to calculate the points of the square. */
    void calculatePoints()
    {
        int radius = theWidth/2;
        pts[0].x = Center.x - radius;
        pts[0].y = Center.y - radius;
        pts[1].x = Center.x - radius;
        pts[1].y = Center.y + radius;
        pts[2].x = Center.x + radius;
        pts[2].y = Center.y + radius;
        pts[3].x = Center.x + radius;
        pts[3].y = Center.y - radius;
        dw.setPoints(pts);
    }
}

/**
 *  Subclass of Shape. Represents a Triangle.
 */
class Triangle extends Shape
{
    /** Drawing window. */
    drawWindow dw;
    /** Base and Height of triange. */
    int Base, Height;
    /** angle of lower left corner. */
    int Angle;

    /** Constructor. */
    Triangle(drawWindow win, Point center, int base, int height,
                             int baseAngle)
    {
        dw = win;
        Center = center;
        Base = base;
        Height = height;
        Angle = baseAngle;
        pts = new Point[3];
        for (int i=0; i < 3; i++)
            pts[i] = new Point(0,0);
    }

    /** Method to calculate the points of the triangle. */
    void calculatePoints()
    {
        int halfBase = Base/2;
        int halfHeight = Height/2;

        // create base
        pts[0].x = Center.x - halfBase;
        pts[0].y = Center.y + halfHeight;
        pts[2].x = Center.x + halfBase;
        pts[2].y = Center.y + halfHeight;

        // now the apex
        if (Angle == 90)
        {
            pts[1].x = Center.x - halfBase;
            pts[1].y = Center.y - halfHeight;
        }
        else if (Angle < 90)
        {
            int adjacent_angle = 90 - Angle;
            int opposite_angle = 180 - (adjacent_angle + 90);
            double radians = ((double)opposite_angle/180.0 * Math.PI);
            double theTan = Math.tan(radians);
            pts[1].x = (int) (pts[0].x +
                       ((double)(Height)/theTan));
            pts[1].y = Center.y - halfHeight;
        }
        else if (Angle > 90)
        {
            int adjacent_angle = 180 - Angle;
            double radians = (double)adjacent_angle/180.0 * Math.PI;
            double theTan = Math.tan(radians);
            pts[1].x = (int) (pts[0].x -
                       ((double)(Height)/theTan));
            pts[1].y = Center.y - halfHeight;
        }
        dw.setPoints(pts);
    }
}

/**
 * Subclass of Shape.  Represents a Hexagon.
 */
class Hexagon extends Shape
{
    /** The drawing window. */
    drawWindow dw;
    /** Base and Height of triangle. */
    int triangleBase, triangleHeight;

    /** constructor. */
    Hexagon (drawWindow win, Point center, int base, int height)
    {
        dw = win;
        Center = center;
        triangleBase = base;

        // *** triangle height is half the height of the Hex
        triangleHeight = height;

        pts = new Point[6];
        for (int i=0; i < pts.length; i++)
            pts[i] = new Point(0,0);
    }

    /** Method to calculate the points of the hexagon. */
    void calculatePoints()
    {
        int halfBase = triangleBase/2;

        // upper and lower side are trivial
        pts[0].x = Center.x - halfBase;
        pts[0].y = Center.y - triangleHeight;
        pts[5].x = Center.x + halfBase;
        pts[5].y = Center.y - triangleHeight;

        pts[2].x = Center.x - halfBase;
        pts[2].y = Center.y + triangleHeight;
        pts[3].x = Center.x + halfBase;
        pts[3].y = Center.x + triangleHeight;

        // get points to connect sides
        double radians = 60.0/180.0 * Math.PI;
        double theTan = Math.tan(radians);
        int right_triangle_adjacent = (int)
                        ((double)triangleHeight/theTan);
        pts[1].x = pts[0].x - right_triangle_adjacent;
        pts[1].y = Center.y;
        pts[4].x = pts[5].x + right_triangle_adjacent;
        pts[4].y = Center.y;
        dw.setPoints(pts);
    }
}

/**
 * Class to draw the shapes in a window frame.
 */
class drawWindow extends Frame implements ActionListener
{
    /** Points of the shape to draw. */
    private Point dPts[];
    /** Random number. */
    Random dice;

    /** Constructor. */
    drawWindow()
    {
        super("draw Shapes");
        setBackground(Color.white);
        setForeground(Color.red);

        // initialize the random number seed
        dice = new Random();

        // add a menuBar
        MenuBar mBar = new MenuBar();
        setMenuBar(mBar);
        Menu FileMenu = new Menu("File");
        MenuItem itemRef = new MenuItem("Quit");
        itemRef.addActionListener(this);
        FileMenu.add(itemRef);
        mBar.add(FileMenu);
        Menu ShapeMenu = new Menu("Shapes");
        itemRef = new MenuItem("Square");
        itemRef.addActionListener(this);
        ShapeMenu.add(itemRef);
        itemRef = new MenuItem("Triangle");
        itemRef.addActionListener(this);
        ShapeMenu.add(itemRef);
        itemRef = new MenuItem("Hexagon");
        itemRef.addActionListener(this);
        ShapeMenu.add(itemRef);
        mBar.add(ShapeMenu);

        // size and show
        setSize(200,200);
        setLocation(new Point(50,50));
        setVisible(true);
    }

    /** Method to implement ActionListener. */
    public void actionPerformed(ActionEvent evt)
    {
        String command = evt.getActionCommand();
        if ("Quit".equals(command))
        {
            dispose();
            System.exit(0);
        }
        else if ("Square".equals(command))
        {
            setPoints(null);
            repaint(); // clear window

            // roll a random # between 10 and 100
            int roll = dice.nextInt() % 100;
            if (roll < 0) roll = -roll;
            if (roll < 10) roll = 10;

            Square aSquare = new Square(this,new Point(100,100),roll);
            aSquare.calculatePoints();
            paint(getGraphics());
        }
        else if ("Triangle".equals(command))
        {
            setPoints(null);
            repaint(); // clear window

            // roll 2 random #'s between 10 and 100
            int base = dice.nextInt() % 100;
            if (base < 0) base = -base;
            if (base < 10) base = 10;

            int height = dice.nextInt() % 100;
            if (height < 0) height = -height;
            if (height < 10) height = 10;

            // roll a random angle between 1 and 179
            int angle = dice.nextInt() % 179;
            if (angle < 0) angle = -angle;
            if (angle < 1) angle = 1;

            Triangle aTriangle =
                    new Triangle(this, new Point(100,100),
                                    base, height, angle);
            aTriangle.calculatePoints();
            paint(getGraphics());
        }
        else if ("Hexagon".equals(command))
        {
            setPoints(null);
            repaint(); // clear window

            // roll a random #'s between 10 and 100
            int base = dice.nextInt() % 100;
            if (base < 0) base = -base;
            if (base < 10) base = 10;

            /* calculate the height of the
                    equilateral triangle  */
            int right_triangle_adjacent = base/2;
            double radians = 60.0/180.0 * Math.PI;
            double theTan = Math.tan(radians);

            int equilateral_height = (int)
                            (theTan * right_triangle_adjacent);

            Hexagon aHexagon =
                    new Hexagon(this, new Point(100,100),
                                    base, equilateral_height);
            aHexagon.calculatePoints();
            paint(getGraphics());
        }
    }

    /** Method to set the points in the drawing window. */
    void setPoints(Point inPoints[])
    {
        dPts = inPoints;
    }

    /** Method to paint the window. */
    public void paint(Graphics g)
    {
        if (dPts != null)
        {
            for (int i=0; i < dPts.length - 1; i++)
                    g.drawLine(dPts[i].x, dPts[i].y,
                               dPts[i+1].x, dPts[i+1].y);

            g.drawLine(dPts[dPts.length-1].x,
                       dPts[dPts.length-1].y,
                       dPts[0].x,dPts[0].y);
        }
    }
}

/**
 * Wrapper class to demonstrate abstract classes.
 */
class DrawShapes
{
    /** main() method to invoke from JVM. */
    public static void main(String args[])
    {
        new drawWindow();
    }
}
