/** RobotBattle.java */
package jwiley.chp4;

import java.io.File;
import java.util.Random;

public class RobotBattle
{
    Robot [] robots;
    String sPackageName = "jwiley.chp4.Robots";
    Random dice = new Random();

    Robot [] getRobots()
    { return robots; }

    public void loadRobots()
    {
        String sCurrentDir = System.getProperty("user.dir");
        File fDir = new File(sCurrentDir + File.separator + "Robots");
        String [] sFileList = fDir.list();

        int iRobotCount=0;
        for (int i=0; i < sFileList.length; i++)
        {
            if (sFileList[i].endsWith(".class"))
                iRobotCount++;
        }

        robots = new Robot[iRobotCount];
        int iCnt=0;
        for (int i=0; i < sFileList.length; i++)
        {
            if (sFileList[i].endsWith(".class"))
            {
                try
                {
                    String sClassName = sPackageName + "." +
                                        sFileList[i].substring(0,
                                                     sFileList[i].length() - 6);
                    System.out.println("Loading " + sClassName + "...");
                    Class robotClass = Class.forName(sClassName);
                    robots[iCnt++] = (Robot) robotClass.newInstance();
                } catch (Exception e)
                  {
                    System.out.println("Failed to instantiate class from " +
                                        sFileList[i]);
                    continue;
                  }
            }
        }
    }

    int getRobotsAssessment(Robot r)
    {
        return r.getFirePower() + r.getSpeed() + r.getArmor();
    }

    boolean coinToss()
    {
        int iNum = Math.abs(dice.nextInt() % 100);
        return (iNum > 50);
    }

    public static void main(String args[])
    {
        // get initial Robot building funds from command line
        if (args.length < 1)
        {
            System.out.println("USAGE: java RobotBattle #\n The # represents" +
                               " initial robot building funds.");
            System.exit(1);
        }

        int iFunds = Integer.parseInt(args[0]);

        RobotBattle ww3 = new RobotBattle();
        ww3.loadRobots();

        // get the robots
        Robot [] contestants = ww3.getRobots();

        // initialize the robots
        for (int i=0; i < contestants.length; i++)
            contestants[i].setInitialCapital(iFunds);

        // introduce the contestants
        System.out.println("Introducing the contestants...");
        for (int i=0; i < contestants.length; i++)
            System.out.println(contestants[i].toString());

        // make the first two fight
        if (contestants.length < 2)
        {
            System.out.println("Need two robots to fight!");
            System.exit(1);
        }

        Robot robot1 = contestants[0];
        Robot robot2 = contestants[1];

        while (robot1.getArmor() > 0 && robot2.getArmor() > 0)
        {
            robot1.assessOpponent(ww3.getRobotsAssessment(robot2));
            robot2.assessOpponent(ww3.getRobotsAssessment(robot1));

            int iAction1 = robot1.getNextAction();
            int iAction2 = robot2.getNextAction();
            switch (iAction1)
            {
                case Robot.ACTION_OFFENSIVE_ATTACK:
                    boolean handToHand = false;
                    System.out.println(robot1.getName() + " Attacks!");
                    if (robot1.getFirePower() > 0)
                    {
                        System.out.println(robot1.getName() + "'s weapons blaze!");
                        robot1.setFirePower(robot1.getFirePower() - 20);
                    }
                    else
                    {
                        System.out.println(robot1.getName() + " uses hand " +
                                           "to hand combat!");
                        handToHand = true;
                    }

                    if (iAction2 == Robot.ACTION_DEFENSIVE_BLOCK)
                    {
                        System.out.println(robot2.getName() + " BLOCKED IT!");
                    }
                    else if (iAction2 == Robot.ACTION_FLEE)
                    {
                        // check if successful
                        if (ww3.coinToss())
                            System.out.println(robot2.getName() + " Dodged it!");
                        else
                        {
                            System.out.println(robot2.getName() + " got grazed!");
                            robot2.setArmor(robot2.getArmor() - 10);
                        }
                    }
                    else // BOTH ATTACK
                    {
                        // gets hit?
                        if (ww3.coinToss())
                        {
                            System.out.println(robot1.getName() + " SCORED a DIRECT HIT!");
                            if (handToHand)
                                robot2.setArmor(robot2.getArmor() - 2);
                            else
                                robot2.setArmor(robot2.getArmor() - 50);
                        }
                        else
                        {
                            // missed - return FIRE!
                            System.out.println(robot1.getName() + " MISSED!");
                            int iReturnFire = 0;
                            if (robot2.getFirePower() > 0)
                            {
                                System.out.println(robot2.getName() + " RETURNS FIRE!");
                                iReturnFire = 50;
                                robot2.setFirePower(robot2.getFirePower() - 20);
                            }
                            else
                            {
                                System.out.println(robot2.getName() + " Fights hand to hand!");
                                iReturnFire = 2;
                            }

                            // check for return fire hit
                            if (ww3.coinToss())
                            {
                                System.out.println(robot2.getName() + " LANDED a blow!");
                                robot1.setArmor(robot1.getArmor() - iReturnFire);
                            }
                        }
                    }
                break;
                case Robot.ACTION_DEFENSIVE_BLOCK:
                    System.out.println(robot1.getName() + " Defensive block.");
                    // logic left as exercise
                break;
                case Robot.ACTION_FLEE:
                    System.out.println(robot1.getName() + " Fleeing...");
                    // logic left as exercise
                break;
                default:
                    System.out.println("Unknown action: " + iAction1);
            }

            // stats
            System.out.println("STATE: " + robot1.toString());
            System.out.println("STATE: " + robot2.toString());
        }

        if (robot1.getArmor() <= 0)
            System.out.println(robot2.getName() + " has WON!");
        else
            System.out.println(robot1.getName() + " has WON!");
    }
}