
package jwiley.chp7;

import java.awt.*;

public class SimplePositioner extends Canvas
{
    Point       pOldPosition = new Point(0,0);
    Point       pNewPosition = new Point(0,0);
    Rectangle   rectMarkerSize;
    Color       colorBack = Color.white;
    Color       colorMarker = Color.black;

    Graphics    offScreenGraphics;
    Image       offScreenImage;

    public SimplePositioner (Point pInitPosition, Rectangle rectMarkerSize)
    {
        if (pInitPosition.x < 0)
            pInitPosition.x = 0;
        else if (pInitPosition.x > 100)
            pInitPosition.x = 100;

        if (pInitPosition.y < 0)
            pInitPosition.y = 0;
        else if (pInitPosition.y > 100)
            pInitPosition.y = 100;

        this.rectMarkerSize = rectMarkerSize;
        setBackground(colorBack);

        pOldPosition.x = pInitPosition.x;
        pOldPosition.y = pInitPosition.y;
    }

    public void paint (Graphics g)
    {
        if (offScreenImage != null)
        {
            g.drawImage(offScreenImage, 0,0,this);
        }
        else
        {
            setMarker(pOldPosition.x, pOldPosition.y);
            draw(pNewPosition, g);
            pOldPosition.x = pNewPosition.x;
            pOldPosition.y = pNewPosition.y;
        }

    }

    public void update (Graphics g)
    {
        if (g != null && offScreenGraphics != null)
        {
            if ((pOldPosition.x != pNewPosition.x) || (pOldPosition.y != pNewPosition.y))
            {
                offScreenGraphics.setXORMode(colorBack);
                draw(pOldPosition, offScreenGraphics);
                offScreenGraphics.setPaintMode();
                offScreenGraphics.setColor(colorMarker);
                draw(pNewPosition, offScreenGraphics);
                pOldPosition.x = pNewPosition.x;
                pOldPosition.y = pNewPosition.y;
            }
            else
            {
                draw(pOldPosition, offScreenGraphics);
            }

            paint(g);
        }
        else
            createOffScreenGraphics();
    }

    public void createOffScreenGraphics()
    {
        Dimension d = getSize();
        offScreenImage = createImage(d.width, d.height);
        offScreenGraphics = offScreenImage.getGraphics();
        draw(pOldPosition, offScreenGraphics);
    }

    public void draw (Point pPosition, Graphics g)
    {
        int iXPosition = pPosition.x - (int)(rectMarkerSize.width / 2);
        int iYPosition = pPosition.y - (int)(rectMarkerSize.height / 2);
        g.fillRect(iXPosition, iYPosition, rectMarkerSize.width, rectMarkerSize.height);
    }

    public void setSize(Dimension d)
    {
        setSize(d.width, d.height);
    }

    public void setSize(int x, int y)
    {
        super.setSize(x, y);
        createOffScreenGraphics();
    }

    public void setMarker (int iXPercent, int iYPercent)
    {
        if (iXPercent < 0)
            iXPercent = 0;
        else if (iXPercent > 100)
            iXPercent = 100;

        if (iYPercent < 0)
            iYPercent = 0;
        else if (iYPercent > 100)
            iYPercent = 100;

        pNewPosition.x = (int)(this.getSize().width * iXPercent * .01);
        pNewPosition.y = (int)(this.getSize().height * iYPercent * .01);
        repaint();
    }

    public static void main (String args[])
    {
        Frame f = new Frame();
        SimplePositioner sp = new SimplePositioner(new Point(50,50), new Rectangle(6,6));
        f.setLayout(new BorderLayout());
        f.add("Center", sp);
        f.setSize(200,200);
        f.show();

        try {   Thread.sleep(2000);  } catch (Exception e)   {   }

        for (int iCount = 0; iCount < 100; iCount++)
        {
            try {   Thread.sleep(30);  } catch (Exception e)   {   }
            sp.setMarker(iCount, iCount);
        }

        for (int iCount = 100; iCount > 0; iCount--)
        {
            try {   Thread.sleep(30);  } catch (Exception e)   {   }
            sp.setMarker(iCount, 100 - iCount);
        }
    }
}
