// Copyright (c) 1995 by Michael C. Daconta
package jwiley.chp8.database;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.InputStream;
import java.util.Vector;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;

/** the single all-encompassing DB class
   will allow multiple DB's to be open simultaneously.
   This will be important when you scale up to a DB server. */
public class DB
{
    /** input stream. */
    static BufferedReader br = new BufferedReader(
                               new InputStreamReader(System.in));
    /** database schema. */
    protected Schema theSchema;
    /** database header record. */
    protected DbHeader theHeader;
    /** database files. */
    protected DbFiles theFiles;
    /** number of records in this database .*/
    protected long recordCount;

    /** mutator. */
    public void setSchema(Schema inSchema)
    { theSchema = inSchema; }

    /** mutator. */
    public void setHeader(DbHeader inHeader)
    {
            theHeader = inHeader;
            recordCount = inHeader.getnum_db_entries();
    }

    /** mutator. */
    public void setdbFiles(DbFiles inFiles)
    { theFiles = inFiles; }

    /** accessor. */
    public Schema getSchema() { return theSchema; }
    /** accessor. */
    public DbHeader getHeader() { return theHeader; }
    /** accessor. */
    public DbFiles getdbFiles() { return theFiles; }

    /** method to create a database record. */
    public synchronized Vector createRecord()
    {
            Vector fldTypes = theHeader.getfldTypes();
            Vector fldNames = theHeader.getfldNames();
            Vector RecVals = new Vector(3);

            for (int i=0; i < fldTypes.size(); i++)
            {
                    System.out.print((String)fldNames.elementAt(i) +
                                                    " : ");
                    System.out.flush();
                    String buf = null;
                    try {
                        buf = br.readLine();
                    } catch (IOException ioe)
                      {
                         System.out.println(ioe.getMessage());
                         return null;
                      }

                    try {
                        switch (((Integer)fldTypes.elementAt(i)).intValue()) {
                            case DbHeader.STRING_TYPE:
                                    RecVals.addElement(new String(buf));
                                    break;
                            case DbHeader.LONG_TYPE:
                                    Long num = Long.valueOf(buf);
                                    RecVals.addElement(num);
                                    break;
                            case DbHeader.DOUBLE_TYPE:
                                     Double aDoub = Double.valueOf(buf);
                                     RecVals.addElement(aDoub);
                                    break;
                            default:
                                    System.out.println("Unknown Type!");
                                    return null;
                        }
                    } catch (Exception e)
                    {
                        System.out.println(e.getMessage());
                        return null;
                    }
            }
            return RecVals;
    }

    /** method to display a record. */
    public synchronized void displayRecord(Vector RecVals)
    {
            Vector fldTypes = theHeader.getfldTypes();
            Vector fldNames = theHeader.getfldNames();

            for (int i=0; i < fldTypes.size(); i++)
            {
                    System.out.println((String)fldNames.elementAt(i) +
                                                    " : " +
                                        RecVals.elementAt(i));
            }
    }

    /** method to convert record values to a byte array suitable
        for storage. */
    public synchronized byte [] RecValsToByteArray(Vector RecVals)
    {
        if (RecVals == null)
            return null;

        ByteArrayOutputStream bos;
        try {
            bos = new ByteArrayOutputStream();
            DataOutputStream dos = new DataOutputStream(bos);
            Vector fldTypes = theHeader.getfldTypes();

            for (int i=0; i < fldTypes.size(); i++)
            {
                    switch (((Integer)fldTypes.elementAt(i)).intValue()) {
                            case DbHeader.STRING_TYPE:
                                    dos.writeUTF((String)RecVals.elementAt(i));
                                    break;
                            case DbHeader.LONG_TYPE:
                                    dos.writeLong(((Long)RecVals.elementAt(i)).longValue());
                                   break;
                            case DbHeader.DOUBLE_TYPE:
                                    dos.writeDouble(((Double)RecVals.elementAt(i)).doubleValue());
                                    break;
                            default:
                                    System.out.println("Unknown Type!");
                                    return null;
                    }
            }

        } catch (Exception e)
          {
             System.out.println(e.getMessage());
             return null;
          }

       return bos.toByteArray();
    }

    /** method to convert a byte array into a Vector or record
        values. */
    public synchronized Vector ByteArrayToRecVals(byte bytes[])
    {
        Vector RecVals = new Vector(3);
        ByteArrayInputStream bis = new ByteArrayInputStream(bytes);
        DataInputStream dis = new DataInputStream(bis);

        try {
            Vector fldTypes = theHeader.getfldTypes();

            for (int i=0; i < fldTypes.size(); i++)
            {
                    switch (((Integer)fldTypes.elementAt(i)).intValue()) {
                            case DbHeader.STRING_TYPE:
                                    RecVals.addElement(new String(dis.readUTF()));
                                    break;
                            case DbHeader.LONG_TYPE:
                                    RecVals.addElement(new Long(dis.readLong()));
                                   break;
                            case DbHeader.DOUBLE_TYPE:
                                    RecVals.addElement(new Double(dis.readDouble()));
                                    break;
                            default:
                                    System.out.println("Unknown Type!");
                                    return null;
                    }
            }

        } catch (Exception e)
          {
             System.out.println(e.getMessage());
             return null;
          }

       return RecVals;
    }

    /** initialize (create or open) a Database */
    public void initDBfiles()
    {
            theFiles = new DbFiles(theHeader.getdbname());

            if (theFiles.getdataFileLength() == 0)
            {
               // INIT the DB
               /* theHeader is stored in the data file but
                  is ALWAYS the first record in the idx file. */
               byte headerBytes[] = theHeader.toByteArray();

               // write the header to the data file at 0 idxPos
               if (theFiles.writeBytes(headerBytes, 0, 0))
               {
                    System.out.println("Database successfully initialized.");
               }
               else
               {
                    System.out.println("Failed to initialize DB.");
                    System.exit(1);
               }
            }
            else
            {
               // OPEN DB and retrieve header
               byte headerBytes[] = theFiles.readBytes(0);

               if (headerBytes != null)
               {
                    theHeader = new DbHeader(headerBytes);
                    recordCount = theHeader.getnum_db_entries();
                    System.out.println("Database successfully opened.");
               }
               else
               {
                    System.out.println("Failed to open the DB.");
                    System.exit(1);
               }
            }
    }

    /** close the database. */
    public void close()
    {
            if (theFiles != null)
            {
                    /* write the current header out to
                       the data file. */
                    theFiles.writeHeader(theHeader);
                    theFiles.close();
            }
    }

    /** add a record to the database. */
    public boolean addRecord(Vector RecVals)
    {
            // convert vector to bytes
            byte dataBytes[] = RecValsToByteArray(RecVals);
            if (dataBytes == null)
                    return false;

            /* write the bytes to the DbFiles at the
               EOF  **** !!! modify after delete!!!!!!!!! */

            if (theFiles == null)
                    initDBfiles();

            if (!theFiles.writeBytes(dataBytes,
                                     theFiles.getdataFileLength(),
                                     theFiles.getidxFileLength()))
                 return false;

            recordCount++;
            theHeader.setnum_db_entries(recordCount);
            return true;
    }

    /** view all records in the database. */
    public void viewRecords()
    {
            if (theFiles == null)
                    initDBfiles();

            if (recordCount > 0)
            {
                    boolean done = false;
                    System.out.println("> " + recordCount +
                                       " records in the DB.");
                    // note first idxRec for header
                    long idxPos = IdxRecord.REC_LENGTH;
                    for (int i=0; i < recordCount; i++)
                    {
                            byte dataBytes[] = theFiles.readBytes(idxPos);
                            if (dataBytes != null)
                            {
                                    // convert to vector
                                    Vector RecVals = ByteArrayToRecVals(dataBytes);
                                    displayRecord(RecVals);
                                    System.out.println("Hit Return to continue or" +
                                                       " 99 to return to main menu.");
                                    String buf=null;
                                    try {
                                          buf = br.readLine();
                                    } catch (Exception e) { }
                                    if (buf != null && buf.equals("99"))
                                            break;
                            }
                            else
                            {
                                    System.out.println("Error reading Database.");
                                    break;
                            }

                            idxPos += IdxRecord.REC_LENGTH;
                    }
            }
            else
                    System.out.println("No records in DB.");
    }
}
