// Copyright (c) 1995 by Michael C. Daconta
package jwiley.chp8.database;

import java.util.Vector;
import java.io.ByteArrayOutputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.DataOutputStream;
import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.io.FileInputStream;
import java.util.StringTokenizer;

public class DbHeader implements LightPersistence
{
        public static final int STRING_TYPE = 1;
        public static final int LONG_TYPE = 2;
        public static final int DOUBLE_TYPE = 3;

        protected String dbname;
        protected int numFields;
        protected Vector fldTypes; // ints
        protected Vector fldNames; // Strings
        protected long free_list_rec;
        protected long free_list_length;
        protected long num_fl_entries;
        protected long num_db_entries;

        // accessors
        public String getdbname() { return dbname; }
        public int getnumFields() { return numFields; }
        public Vector getfldTypes() { return fldTypes; }
        public Vector getfldNames() { return fldNames; }
        public long getfree_list_rec() { return free_list_rec; }
        public long getfree_list_length() { return free_list_length; }
        public long getnum_fl_entries() { return num_fl_entries; }
        public long getnum_db_entries() { return num_db_entries; }

        // mutators
        public void setfree_list_rec(long inRec)
        { free_list_rec = inRec; }
        public void setfree_list_length(long inLength)
        { free_list_length = inLength; }
        public void setnum_fl_entries(long inNum)
        { num_fl_entries = inNum; }
        public void setnum_db_entries(long inNum)
        { num_db_entries = inNum; }

        static int translateType(String inStr)
        {
                if (inStr.equals("String"))
                        return 1;
                else if (inStr.equals("Long"))
                        return 2;
                else if (inStr.equals("Double"))
                        return 3;
                else
                        return -1;
        }

        public DbHeader(String schemaFile)
        {
            // get dbname from schemafile name
            int dot = schemaFile.indexOf('.');
            dbname = schemaFile.substring(0,dot);

            fldTypes = new Vector(3);
            fldNames = new Vector(3);

            // parse the schema file and create the header
            try {
                FileInputStream fis = new FileInputStream(schemaFile);
                BufferedReader br = new BufferedReader(
                                    new InputStreamReader(fis));

                int fldCount=0;
                // each line with 2 tokens is a field
                String Line;

                while ( (Line = br.readLine()) != null)
                {
                        StringTokenizer theToks = new StringTokenizer(Line," ");
                        if (theToks.countTokens() == 2)
                        {
                                fldCount++;
                                String fldName = theToks.nextToken();
                                String fldTypeStr = theToks.nextToken();
                                int fldType = translateType(fldTypeStr);

                                // store in Vectors
                                fldTypes.addElement(new Integer(fldType));
                                fldNames.addElement(new String(fldName));
                        }
                }

                numFields = fldCount;
                free_list_rec = 0;
                free_list_length = 0;
                num_fl_entries = 0;
                num_db_entries = 0;

            } catch (Exception e)
              {
                System.out.println(e.getMessage());
              }
        }

        public DbHeader(byte bytes[])
        {
                // translate a byte array into a header
                ByteArrayInputStream bis = new ByteArrayInputStream(bytes);
                DataInputStream dis = new DataInputStream(bis);

                try {
                        dbname = dis.readUTF();
                        numFields = dis.readInt();
                        fldTypes = new Vector(3);
                        fldNames = new Vector(3);

                        for (int i=0; i < numFields; i++)
                                fldTypes.addElement(new Integer(dis.readInt()));

                        for (int i=0; i < numFields; i++)
                                fldNames.addElement(new String(dis.readUTF()));

                        free_list_rec = dis.readLong();
                        free_list_length = dis.readLong();
                        num_fl_entries = dis.readLong();
                        num_db_entries = dis.readLong();
                } catch (Exception e)
                  {
                        System.out.println("Could not translate header.");
                        System.out.println(e.getMessage());
                        return;
                  }
        }

        public byte [] toByteArray()
        {
                ByteArrayOutputStream bos = new ByteArrayOutputStream();
                DataOutputStream dos = new DataOutputStream(bos);

                try {
                        dos.writeUTF(dbname);
                        dos.writeInt(numFields);

                        for (int i=0; i < numFields; i++)
                                dos.writeInt(((Integer)fldTypes.elementAt(i)).intValue());

                        for (int i=0; i < numFields; i++)
                                dos.writeUTF((String)fldNames.elementAt(i));

                        dos.writeLong(free_list_rec);
                        dos.writeLong(free_list_length);
                        dos.writeLong(num_fl_entries);
                        dos.writeLong(num_db_entries);
                } catch (Exception e)
                  {
                        System.out.println(e.getMessage());
                        return null;
                  }

                return bos.toByteArray();
        }
}
