// Copyright (c) 1995 by Michael C. Daconta
package jwiley.chp8.database;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;

/** index into the database. An index consists of
    the position and length of a variable-lengthed
    database record. */
public class IdxRecord implements LightPersistence
{
    /** record length. Always a fixed size. */
    public static final int REC_LENGTH = 16; // 16 bytes
    /** record position in .dat file. */
    protected long recPos;
    /** length of data record in .dat file. */
    protected long recLength;

    /** constructor. */
    public IdxRecord(long inPos, long inLength)
    {
            recPos = inPos;
            recLength = inLength;
    }

    /** accessor. */
    public long getrecPos() { return recPos; }
    /** accessor. */
    public long getrecLength() { return recLength; }

    /** construct from a byte array. */
    public IdxRecord(byte bytes[])
    {
            // translate a byte array into a header
            ByteArrayInputStream bis = new ByteArrayInputStream(bytes);
            DataInputStream dis = new DataInputStream(bis);

            try {
                    recPos = dis.readLong();
                    recLength = dis.readLong();
            } catch (Exception e)
              {
                    System.out.println(e.getMessage());
                    return;
              }
    }

    /** convert to a byte array. */
    public byte [] toByteArray()
    {
            ByteArrayOutputStream bos = new ByteArrayOutputStream();
            DataOutputStream dos = new DataOutputStream(bos);

            try {
                    dos.writeLong(recPos);
                    dos.writeLong(recLength);
            } catch (Exception e)
              {
                    System.out.println(e.getMessage());
                    return null;
              }

            return bos.toByteArray();
    }
}

