// Copyright (c) 1995 by Michael C. Daconta
package jwiley.chp8.database;
import java.util.Vector;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.FileOutputStream;
import java.io.PrintWriter;

/** Class which stores the database schema. That is the
    definition of this table. */
public class Schema
{
    /** vector of strings. */
    protected Vector strings;
    /** filename to store schema in. */
    protected String fileName;

    /** accessor. */
    public String getFileName()
    { return fileName; }

    /** interactive routine to build a schema from user input. */
    public boolean buildSchema()
    {
            BufferedReader br = new BufferedReader(
                                new InputStreamReader(System.in));
            System.out.println("Definining a database consists of two steps:");
            System.out.println("1) Name the database.");
            System.out.println("2) Describe the fields of the database.");
            System.out.println(" ");
            System.out.println("Hit Return to continue, else type 99 to return to main menu.");
            String buf=null;
            try {
                    buf = br.readLine();
            } catch (Exception e) { return false; }
            if (buf.equals("99"))
                    return false;

            // get the dbname
            System.out.print("Enter the database name: ");
            System.out.flush();
            try { buf = br.readLine(); } catch (Exception e) { return false; }

            if (buf.length() == 0)
            {
                    System.out.println("Must have a database name to define a DB.");
                    return false;
            }

            buf.replace(' ', '_');
            // store the filename
            fileName = new String(buf + ".schema");

            System.out.println(" ");
            System.out.println("To describe the fields in your database.");
            System.out.println("There are two pieces of information for each field.");
            System.out.println("1) The field name.  This is what you call the data,");
            System.out.println("   for example, an address database would have names");
            System.out.println("   like - name, street, city...");
            System.out.println("2) The data type.  This is what type of data the field");
            System.out.println("   holds.  There are currently only three (3) choices:");
            System.out.println("   string (text), number (whole numbers), decimal.");

            boolean finished = false;
            strings = new Vector(3);

            while (!finished)
            {
                String fldName=null, fldType=null;
                boolean good = false;
                while (!good)
                {
                        System.out.println(" ");
                        System.out.print("Enter the field name: ");
                        System.out.flush();
                        try {
                                fldName = br.readLine();
                        } catch (Exception e) { return false; }
                        if (fldName.length() > 0)
                                good = true;
                }

                // replace any blanks in the string
                fldName.replace(' ', '_');

                good = false;

                while (!good)
                {
                        System.out.println("Choose the field type:");
                        System.out.println("1) String (text - alpha and numbers)");
                        System.out.println("2) Long (whole numbers)");
                        System.out.println("3) Double (decimals)");
                        System.out.print("Enter choice: ");
                        System.out.flush();
                        String choiceStr;
                        try {
                                choiceStr = br.readLine();
                        } catch (Exception e) { return false; }
                        int choice = Integer.parseInt(choiceStr);
                        switch (choice) {
                                case 1:
                                        fldType = new String("String");
                                        good = true;
                                        break;
                                case 2:
                                        fldType = new String("Long");
                                        good = true;
                                        break;
                                case 3:
                                        fldType = new String("Double");
                                        good = true;
                                        break;
                                default:
                                        System.out.println("Not a valid option!");
                        }
                }

                // add the field data
                strings.addElement(new String(fldName + " " +  fldType));

                System.out.println("Hit Return to continue, else type 99 when done entering fields.");
                try {
                        buf = br.readLine();
                } catch (Exception e) { return false; }
                if (buf.equals("99"))
                        finished = true;
            }

            return true;
    }

    /** write schema to output file. */
    public void outputSchema()
    {
        try
        {
             FileOutputStream fos = new FileOutputStream(fileName);
             PrintWriter pw = new PrintWriter(fos, true);

             for (int i=0; i < strings.size(); i++)
                    pw.println(((String)strings.elementAt(i)));

             fos.close();
        } catch (Exception e)
          {
             System.out.println("Unable to store Schema in " + fileName);
             System.out.println("Re-Enter schema with different filename.");
             return;
          }
    }
}
